@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
       None
@@ -219,7 +220,7 @@ ffebad_start_ (bool lex_override, ffebad errnum, ffebadSeverity sev,
 	break;
       }
   }
-#endif
+#endif	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
 
   ffebad_is_temp_inhibited_ = FALSE;
   ffebad_errnum_ = errnum;
@@ -257,15 +258,14 @@ ffebad_here (ffebadIndex index, ffewhereLine line, ffewhereColumn col)
     return;
 
   assert (index < FFEBAD_MAX_);
-  if (ffewhere_line_is_unknown (line))
+  ffebad_here_[index].line = ffewhere_line_use (line);
+  ffebad_here_[index].col = ffewhere_column_use (col);
+  if (ffewhere_line_is_unknown (line)
+      || ffewhere_column_is_unknown (col))
     {
-      ffebad_here_[index].line = line;
-      ffebad_here_[index].col = ffewhere_column_unknown ();
       ffebad_here_[index].tag = FFEBAD_MAX_;
       return;
     }
-  ffebad_here_[index].line = ffewhere_line_use (line);
-  ffebad_here_[index].col = ffewhere_column_use (col);
   ffebad_here_[index].tag = 0;	/* For now, though it shouldn't matter. */
 
   /* Sort the source line/col points into the order they occur in the source
@@ -421,6 +421,9 @@ ffebad_finish ()
 	{
 	  if (bi != 0)
 	    fputc ('\n', stderr);
+#if FFECOM_targetCURRENT == FFECOM_targetGCC
+	  report_error_function (fn);
+#endif	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
 	  fprintf (stderr,
 #if 0
 		   "Line %" ffewhereLineNumber_f "u of %s:\n   %s\n   %s%c",
@@ -438,7 +441,7 @@ ffebad_finish ()
 		   pointer);
 	  last_line_num = ln;
 	  last_col_num = cn;
-	  s = "(continued:)";
+	  s = "(continued):";
 	}
       else
 	{
@@ -452,14 +455,16 @@ ffebad_finish ()
     }
   if (ffebad_places_ == 0)
     {
-      char c;
-
       /* Didn't output "warning:" string, capitalize it for message.  */
-      if ((s != NULL) && isalpha (s[0]) && islower (s[0]))
-	c = toupper (s[0]);
-      else
-	c = s[0];
-      fprintf (stderr, "%c%s ", c, &s[1]);
+      if ((s[0] != '\0') && isalpha (s[0]) && islower (s[0]))
+	{
+	  char c;
+
+	  c = toupper (s[0]);
+	  fprintf (stderr, "%c%s ", c, &s[1]);
+	}
+      else if (s[0] != '\0')
+	fprintf (stderr, "%s ", s);
     }
   else
     fputc ('\n', stderr);
