@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.  */
 
 #include "proj.h"
 #include <ctype.h>
@@ -253,7 +254,7 @@ main (int argc, char **argv)
   int cc;
   bool do_exit = FALSE;
 
-  for (i = 0; i < ARRAY_SIZE (names); ++i)
+  for (i = 0; ((size_t) i) < ARRAY_SIZE (names); ++i)
     {				/* Initialize length/name ordered list roots. */
       names[i].first = (name) &names[i];
       names[i].last = (name) &names[i];
@@ -400,8 +401,8 @@ main (int argc, char **argv)
       strcpy (last_buf, newname->name_uc);
 
       /* Append name to end of alpha-sorted list (assumes names entered in
-         alpha order wrt name, not kwname, even though kwname is output from
-         this list). */
+	 alpha order wrt name, not kwname, even though kwname is output from
+	 this list). */
 
       n = names_alpha.last;
       newname->next_alpha = n->next_alpha;
@@ -512,7 +513,7 @@ typedef enum %s_ %s;\n\
 
 /* Now output the length as a case, followed by the binary search within that length.  */
 
-      for (len = 0; len < ARRAY_SIZE (names); ++len)
+      for (len = 0; ((size_t) len) < ARRAY_SIZE (names); ++len)
 	{
 	  if (names[len].first != (name) &names[len])
 	    {
@@ -633,7 +634,7 @@ testname (bool nested, int indent, name first, name last)
   int numhalf;
 
   assert (!nested || indent >= 2);
-  assert (indent + 4 < ARRAY_SIZE (spaces));
+  assert (((size_t) indent) + 4 < ARRAY_SIZE (spaces));
 
   num = 0;
   numhalf = 0;
@@ -707,7 +708,7 @@ testnames (bool nested, int indent, int len, name first, name last)
   int numhalf;
 
   assert (!nested || indent >= 2);
-  assert (indent + 4 < ARRAY_SIZE (spaces));
+  assert (((size_t) indent) + 4 < ARRAY_SIZE (spaces));
 
   num = 0;
   numhalf = 0;
