@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Owning Modules:
       global.c
@@ -92,6 +93,7 @@ void ffeglobal_new_progunit_ (ffesymbol s, ffelexToken t, ffeglobalType type);
 void ffeglobal_new_common (ffesymbol s, ffelexToken t, bool blank);
 void ffeglobal_pad_common (ffesymbol s, ffetargetAlign pad, ffewhereLine wl,
 			   ffewhereColumn wc);
+ffeglobal ffeglobal_promoted (ffesymbol s);
 void ffeglobal_save_common (ffesymbol s, bool save, ffewhereLine wl,
 			    ffewhereColumn wc);
 bool ffeglobal_size_common (ffesymbol s, long size);
@@ -101,10 +103,10 @@ void ffeglobal_terminate_1 (void);
 
 #if FFECOM_targetCURRENT == FFECOM_targetFFE
 #define FFEGLOBAL_ENABLED 0
-#else
-#if FFECOM_targetCURRENT == FFECOM_targetGCC
+#elif FFECOM_targetCURRENT == FFECOM_targetGCC
 #define FFEGLOBAL_ENABLED 1
-#endif
+#else
+#error
 #endif
 
 #define ffeglobal_common_init(g) ((g)->init != 0)
