@@ -1,5 +1,5 @@
 /* intrin.h -- Public interface for intrin.c
-   Copyright (C) 1995 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
    Contributed by James Craig Burley (burley@gnu.ai.mit.edu).
 
 This file is part of GNU Fortran.
@@ -24,18 +24,21 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #ifndef _H_f_intrin
 #define _H_f_intrin
 
+#ifndef FFEINTRIN_DOC
+#define FFEINTRIN_DOC 0	/* 1 means intrinsic documentation only (intdoc.c). */
+#endif
+
 typedef enum
   {
     FFEINTRIN_familyNONE,	/* Not in any family. */
     FFEINTRIN_familyF77,	/* ANSI FORTRAN 77. */
-    FFEINTRIN_familyASC,	/* ASCII-related (ACHAR, IACHAR). */
-    FFEINTRIN_familyMIL,	/* MIL STD 1753 (MVBITS, etc). */
-    FFEINTRIN_familyDCP,	/* Unnamed DOUBLE COMPLEX intrinsics. */
+    FFEINTRIN_familyGNU,	/* GNU Fortran intrinsics. */
+    FFEINTRIN_familyF2C,	/* f2c intrinsics. */
     FFEINTRIN_familyF90,	/* Fortran 90. */
     FFEINTRIN_familyVXT,	/* VAX/VMS FORTRAN. */
-    FFEINTRIN_familyFVZ,	/* f2c, VAX/VMS DOUBLE COMPLEX intrinsics. */
-    FFEINTRIN_familyF2C,	/* f2c intrinsics. */
-    FFEINTRIN_familyF2Z,	/* f2c DOUBLE COMPLEX intrinsics. */
+    FFEINTRIN_familyMIL,	/* MIL STD 1753 (MVBITS, etc), in mil, vxt, and f90. */
+    FFEINTRIN_familyASC,	/* ASCII-related (ACHAR, IACHAR), both f2c and f90. */
+    FFEINTRIN_familyFVZ,	/* in both f2c and VAX/VMS FORTRAN. */
     FFEINTRIN_familyF2U,	/* libf2c/libU77 UNIX system intrinsics. */
     FFEINTRIN_family,
   } ffeintrinFamily;
@@ -43,7 +46,7 @@ typedef enum
 typedef enum
   {
 #define DEFNAME(UPPER,LOWER,MIXED,GEN,SPEC)
-#define DEFGEN(CODE,NAME,SPEC1,SPEC2) CODE,
+#define DEFGEN(CODE,NAME,SPEC1,SPEC2) FFEINTRIN_gen ## CODE,
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP)
 #define DEFIMP(CODE,NAME,GFRT,CONTROL)
 #define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP)
@@ -60,7 +63,7 @@ typedef enum
   {
 #define DEFNAME(UPPER,LOWER,MIXED,GEN,SPEC)
 #define DEFGEN(CODE,NAME,SPEC1,SPEC2)
-#define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP) CODE,
+#define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP) FFEINTRIN_spec ## CODE,
 #define DEFIMP(CODE,NAME,GFRT,CONTROL)
 #define DEFIMQ(CODE,NAME,GFRT,CONTROL,CGIMP)
 #include "intrin.def"
@@ -88,6 +91,8 @@ typedef enum
     FFEINTRIN_imp
   } ffeintrinImp;
 
+#if !FFEINTRIN_DOC
+
 #include "bld.h"
 #include "info.h"
 
@@ -95,7 +100,8 @@ ffeinfoBasictype ffeintrin_basictype (ffeintrinSpec spec);
 ffeintrinImp ffeintrin_codegen_imp (ffeintrinImp imp);
 ffeintrinFamily ffeintrin_family (ffeintrinSpec spec);
 void ffeintrin_fulfill_generic (ffebld *expr, ffeinfo *info, ffelexToken t);
-void ffeintrin_fulfill_specific (ffebld *expr, ffeinfo *info, ffelexToken t);
+void ffeintrin_fulfill_specific (ffebld *expr, ffeinfo *info,
+				 bool *check_intrin, ffelexToken t);
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
 ffecomGfrt ffeintrin_gfrt (ffeintrinImp imp);
 #endif	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
@@ -107,7 +113,7 @@ void ffeintrin_init_0 (void);
 bool ffeintrin_is_actualarg (ffeintrinSpec spec);
 bool ffeintrin_is_intrinsic (char *name, ffelexToken t, bool explicit,
 			     ffeintrinGen *gen, ffeintrinSpec *spec,
-			     ffeintrinImp *imp, ffeinfoKind *kind);
+			     ffeintrinImp *imp);
 ffeinfoKindtype ffeintrin_kindtype (ffeintrinSpec spec);
 char *ffeintrin_name_generic (ffeintrinGen gen);
 char *ffeintrin_name_implementation (ffeintrinImp imp);
@@ -119,6 +125,8 @@ ffeIntrinsicState ffeintrin_state_family (ffeintrinFamily family);
 #define ffeintrin_terminate_3()
 #define ffeintrin_terminate_4()
 
+#endif	/* !FFEINTRIN_DOC */
+
 /* End of #include file. */
 
 #endif
