@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Owning Modules:
       malloc.c
@@ -29,6 +30,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef _H_f_malloc
 #define _H_f_malloc
 
+#ifndef MALLOC_DEBUG
+#define MALLOC_DEBUG 0	/* 1 means check caller's use of this module. */
+#endif
+
 /* Simple definitions and enumerations. */
 
 typedef enum
@@ -59,8 +64,10 @@ struct _malloc_area_
     mallocArea_ next;
     mallocArea_ previous;
     void *where;
+#if MALLOC_DEBUG
     mallocSize size;
     mallocType_ type;
+#endif
     char name[1];
   };
 
@@ -72,6 +79,8 @@ struct _malloc_pool_
     mallocPool youngest;
     mallocArea_ first;
     mallocArea_ last;
+    unsigned long uses;
+#if MALLOC_DEBUG
     mallocSize allocated;
     mallocSize freed;
     mallocSize old_sizes;
@@ -79,14 +88,13 @@ struct _malloc_pool_
     unsigned long allocations;
     unsigned long frees;
     unsigned long resizes;
-    unsigned long uses;
+#endif
     char name[1];
   };
 
 struct _malloc_root_
   {
     struct _malloc_pool_ malloc_pool_image_;
-    char name[6];
   };
 
 /* Global objects accessed by users of this module. */
@@ -113,46 +121,62 @@ mallocPool malloc_pool_use (mallocPool p);
 void *malloc_resize_ (void *ptr, mallocSize new_size);
 void *malloc_resize_inpool_ (mallocPool pool, mallocType_ type, void *ptr,
 			     mallocSize new_size, mallocSize old_size);
+void malloc_verify_inpool_ (mallocPool pool, mallocType_ type, void *ptr,
+			    mallocSize size);
 
 /* Define macros. */
 
 #define malloc_new_ks(pool,name,size) \
-      malloc_new_inpool_(pool,MALLOC_typeKS_,name,size)
+  malloc_new_inpool_ (pool,MALLOC_typeKS_,name,size)
 #define malloc_new_ksr(pool,name,size) \
-      malloc_new_inpool_(pool,MALLOC_typeKSR_,name,size)
+  malloc_new_inpool_ (pool,MALLOC_typeKSR_,name,size)
 #define malloc_new_kp(pool,name,size) \
-      malloc_new_inpool_(pool,MALLOC_typeKP_,name,size)
+  malloc_new_inpool_ (pool,MALLOC_typeKP_,name,size)
 #define malloc_new_kpr(pool,name,size) \
-      malloc_new_inpool_(pool,MALLOC_typeKPR_,name,size)
+  malloc_new_inpool_ (pool,MALLOC_typeKPR_,name,size)
 #define malloc_new_us(pool,name,size) \
-      malloc_new_inpool_(pool,MALLOC_typeUS_,name,size)
+  malloc_new_inpool_ (pool,MALLOC_typeUS_,name,size)
 #define malloc_new_usr(pool,name,size) \
-      malloc_new_inpool_(pool,MALLOC_typeUSR_,name,size)
+  malloc_new_inpool_ (pool,MALLOC_typeUSR_,name,size)
 #define malloc_new_zks(pool,name,size,z) \
-      malloc_new_zinpool_(pool,MALLOC_typeKS_,name,size,z)
+  malloc_new_zinpool_ (pool,MALLOC_typeKS_,name,size,z)
 #define malloc_new_zksr(pool,name,size,z) \
-      malloc_new_zinpool_(pool,MALLOC_typeKSR_,name,size,z)
+  malloc_new_zinpool_ (pool,MALLOC_typeKSR_,name,size,z)
 #define malloc_new_zkp(pool,name,size,z) \
-      malloc_new_zinpool_(pool,MALLOC_typeKP_,name,size,z)
+  malloc_new_zinpool_ (pool,MALLOC_typeKP_,name,size,z)
 #define malloc_new_zkpr(pool,name,size,z) \
-      malloc_new_zinpool_(pool,MALLOC_typeKPR_,name,size,z)
+  malloc_new_zinpool_ (pool,MALLOC_typeKPR_,name,size,z)
 #define malloc_new_zus(pool,name,size,z) \
-      malloc_new_zinpool_(pool,MALLOC_typeUS_,name,size,z)
+  malloc_new_zinpool_ (pool,MALLOC_typeUS_,name,size,z)
 #define malloc_new_zusr(pool,name,size,z) \
-      malloc_new_zinpool_(pool,MALLOC_typeUSR_,name,size,z)
+  malloc_new_zinpool_ (pool,MALLOC_typeUSR_,name,size,z)
 #define malloc_kill_ks(pool,ptr,size) \
-      malloc_kill_inpool_(pool,MALLOC_typeKS_,ptr,size)
+  malloc_kill_inpool_ (pool,MALLOC_typeKS_,ptr,size)
 #define malloc_kill_ksr(pool,ptr,size) \
-      malloc_kill_inpool_(pool,MALLOC_typeKSR_,ptr,size)
-#define malloc_kill_us(pool,ptr) malloc_kill_inpool_(pool,MALLOC_typeUS_,ptr,0)
-#define malloc_kill_usr(pool,ptr) malloc_kill_inpool_(pool,MALLOC_typeUSR_,ptr,0)
+  malloc_kill_inpool_ (pool,MALLOC_typeKSR_,ptr,size)
+#define malloc_kill_us(pool,ptr) \
+  malloc_kill_inpool_ (pool,MALLOC_typeUS_,ptr,0)
+#define malloc_kill_usr(pool,ptr) \
+  malloc_kill_inpool_ (pool,MALLOC_typeUSR_,ptr,0)
 #define malloc_pool_image() (&malloc_root_.malloc_pool_image_)
 #define malloc_resize_ksr(pool,ptr,new_size,old_size) \
-      malloc_resize_inpool_(pool,MALLOC_typeKSR_,ptr,new_size,old_size)
+  malloc_resize_inpool_ (pool,MALLOC_typeKSR_,ptr,new_size,old_size)
 #define malloc_resize_kpr(pool,ptr,new_size,old_size) \
-      malloc_resize_inpool_(pool,MALLOC_typeKPR_,ptr,new_size,old_size)
+  malloc_resize_inpool_ (pool,MALLOC_typeKPR_,ptr,new_size,old_size)
 #define malloc_resize_usr(pool,ptr,new_size) \
-      malloc_resize_inpool_(pool,MALLOC_typeUSR_,ptr,new_size,0)
+  malloc_resize_inpool_ (pool,MALLOC_typeUSR_,ptr,new_size,0)
+#define malloc_verify_kp(pool,name,size) \
+  malloc_verify_inpool_ (pool,MALLOC_typeKP_,name,size)
+#define malloc_verify_kpr(pool,name,size) \
+  malloc_verify_inpool_ (pool,MALLOC_typeKPR_,name,size)
+#define malloc_verify_ks(pool,ptr,size) \
+  malloc_verify_inpool_ (pool,MALLOC_typeKS_,ptr,size)
+#define malloc_verify_ksr(pool,ptr,size) \
+  malloc_verify_inpool_ (pool,MALLOC_typeKSR_,ptr,size)
+#define malloc_verify_us(pool,ptr) \
+  malloc_verify_inpool_ (pool,MALLOC_typeUS_,ptr,0)
+#define malloc_verify_usr(pool,ptr) \
+  malloc_verify_inpool_ (pool,MALLOC_typeUSR_,ptr,0)
 
 /* End of #include file. */
 
