@@ -15,14 +15,14 @@ extern sig_type signal();
 
 ftnint signal_(sigp, proc) integer *sigp; sig_type proc;
 #else
-#include "signal.h"
+#include <signal.h>
+typedef int (*sig_proc)(int);
 
-ftnint signal_(integer *sigp, sig_type proc)
+ftnint signal_(integer *sigp, sig_proc proc)
 #endif
 {
 	int sig;
 	sig = (int)*sigp;
 
-	signal(sig, proc);
-	return 0;
+	return (ftnint)signal(sig, (sig_type)proc);
 	}
