@@ -1,7 +1,7 @@
 #ifdef KR_headers
 extern FILE *fdopen();
 #else
-#ifdef MSDOS
+#if defined (MSDOS) && !defined (GO32)
 #include "io.h"
 #define close _close
 #define creat _creat
@@ -12,7 +12,7 @@ extern FILE *fdopen();
 #ifdef __cplusplus
 extern "C" {
 #endif
-#ifndef MSDOS
+#if !(defined (MSDOS) && !defined (GO32))
 #ifdef OPEN_DECL
 extern int creat(const char*,int), open(const char*,int);
 #endif
@@ -34,7 +34,7 @@ extern char *mktemp(char*);
 #endif
 
 #ifndef NO_FCNTL
-#include "fcntl.h"
+#include <fcntl.h>
 #endif
 
 #ifndef O_WRONLY
