@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.  */
 
 #include "proj.h"
 #include "type.h"
@@ -30,23 +31,25 @@ ffetype_lookup_kind (ffetype base_type, int kind)
 {
   if ((base_type->kinds_ == NULL)
       || (kind < 0)
-      || (kind >= ARRAY_SIZE (base_type->kinds_->type_)))
+      || (((size_t) kind) >= ARRAY_SIZE (base_type->kinds_->type_)))
     return NULL;
 
   return base_type->kinds_->type_[kind];
 }
 
-ffetype ffetype_lookup_star (ffetype base_type, int star)
+ffetype
+ffetype_lookup_star (ffetype base_type, int star)
 {
   if ((base_type->stars_ == NULL)
       || (star < 0)
-      || (star >= ARRAY_SIZE (base_type->stars_->type_)))
+      || (((size_t) star) >= ARRAY_SIZE (base_type->stars_->type_)))
     return NULL;
 
   return base_type->stars_->type_[star];
 }
 
-ffetype ffetype_new (void)
+ffetype
+ffetype_new (void)
 {
   ffetype type;
 
@@ -61,7 +64,8 @@ ffetype ffetype_new (void)
   return type;
 }
 
-void ffetype_set_kind (ffetype base_type, int kind, ffetype type)
+void
+ffetype_set_kind (ffetype base_type, int kind, ffetype type)
 {
   if (base_type->kinds_ == NULL)
     {
@@ -71,7 +75,7 @@ void ffetype_set_kind (ffetype base_type, int kind, ffetype type)
 	= (ffetype_indexes_) malloc_new_kp (malloc_pool_image (),
 					    "ffetype_indexes_[kinds]",
 					    sizeof (*(base_type->kinds_)));
-      for (i = 0; i < ARRAY_SIZE (base_type->kinds_->type_); ++i)
+      for (i = 0; ((size_t) i) < ARRAY_SIZE (base_type->kinds_->type_); ++i)
 	base_type->kinds_->type_[i] = NULL;
     }
 
@@ -80,7 +84,8 @@ void ffetype_set_kind (ffetype base_type, int kind, ffetype type)
   base_type->kinds_->type_[kind] = type;
 }
 
-void ffetype_set_star (ffetype base_type, int star, ffetype type)
+void
+ffetype_set_star (ffetype base_type, int star, ffetype type)
 {
   if (base_type->stars_ == NULL)
     {
@@ -90,7 +95,7 @@ void ffetype_set_star (ffetype base_type, int star, ffetype type)
 	= (ffetype_indexes_) malloc_new_kp (malloc_pool_image (),
 					    "ffetype_indexes_[stars]",
 					    sizeof (*(base_type->stars_)));
-      for (i = 0; i < ARRAY_SIZE (base_type->stars_->type_); ++i)
+      for (i = 0; ((size_t) i) < ARRAY_SIZE (base_type->stars_->type_); ++i)
 	base_type->stars_->type_[i] = NULL;
     }
 
